# coding:utf-8

from xmanage.systemd.service import sd_service as systemd_service  # noqa:E501,F401

try:
    from xmanage.systemd.path import sd_path as systemd_path  # noqa:F401
except Exception:  # pragma: no cover
    pass  # pragma: no cover



# coding:utf-8

from tabulate import tabulate
from xkits_command import ArgParser
from xkits_command import Command
from xkits_command import CommandArgument
from xkits_command import CommandExecutor

try:
    from xmanage.systemd import systemd_path

    @CommandArgument("path", help="show system and user paths")
    def add_cmd_path(_arg: ArgParser):
        _arg.add_argument(dest="sd_path_titles", type=str, nargs="*",
                          default=[], action="extend", metavar="NAME",
                          choices=systemd_path.titles + [[]],
                          help="the path value with this name is shown")

    @CommandExecutor(add_cmd_path)
    def run_cmd_path(cmds: Command) -> int:
        if not cmds.has_sub(add_cmd_path):
            titles = cmds.args.sd_path_titles
            values = titles if len(titles) > 0 else systemd_path
            tabular = [(n, "\n".join(systemd_path[n])) for n in values]
            tstring = tabulate(tabular, headers=["name", "paths"])
            cmds.stdout(tstring)
        return 0
except Exception:  # pragma: no cover
    pass  # pragma: no cover



# coding:utf-8

from enum import Enum
import os
import shutil
from typing import Dict
from typing import List
from typing import Tuple

from xkits_lib.utils import singleton

CMD_SD_PATH = "systemd-path"


if shutil.which(CMD_SD_PATH):

    @singleton
    class __sd_path:

        class table(Enum):
            TEMPORARY = "temporary"
            TEMPORARY_LARGE = "temporary-large"
            SYSTEM_BINARIES = "system-binaries"
            SYSTEM_INCLUDE = "system-include"
            SYSTEM_LIBRARY_PRIVATE = "system-library-private"
            SYSTEM_LIBRARY_ARCH = "system-library-arch"
            SYSTEM_SHARED = "system-shared"
            SYSTEM_CONFIGURATION_FACTORY = "system-configuration-factory"
            SYSTEM_STATE_FACTORY = "system-state-factory"
            SYSTEM_CONFIGURATION = "system-configuration"
            SYSTEM_RUNTIME = "system-runtime"
            SYSTEM_RUNTIME_LOGS = "system-runtime-logs"
            SYSTEM_STATE_PRIVATE = "system-state-private"
            SYSTEM_STATE_LOGS = "system-state-logs"
            SYSTEM_STATE_CACHE = "system-state-cache"
            SYSTEM_STATE_SPOOL = "system-state-spool"
            USER_BINARIES = "user-binaries"
            USER_LIBRARY_PRIVATE = "user-library-private"
            USER_LIBRARY_ARCH = "user-library-arch"
            USER_SHARED = "user-shared"
            USER_CONFIGURATION = "user-configuration"
            USER_RUNTIME = "user-runtime"
            USER_STATE_CACHE = "user-state-cache"
            USER_STATE_PRIVATE = "user-state-private"
            USER = "user"
            USER_DOCUMENTS = "user-documents"
            USER_MUSIC = "user-music"
            USER_PICTURES = "user-pictures"
            USER_VIDEOS = "user-videos"
            USER_DOWNLOAD = "user-download"
            USER_PUBLIC = "user-public"
            USER_TEMPLATES = "user-templates"
            USER_DESKTOP = "user-desktop"
            SEARCH_BINARIES = "search-binaries"
            SEARCH_BINARIES_DEFAULT = "search-binaries-default"
            SEARCH_LIBRARY_PRIVATE = "search-library-private"
            SEARCH_LIBRARY_ARCH = "search-library-arch"
            SEARCH_SHARED = "search-shared"
            SEARCH_CONFIGURATION_FACTORY = "search-configuration-factory"
            SEARCH_STATE_FACTORY = "search-state-factory"
            SEARCH_CONFIGURATION = "search-configuration"
            SYSTEMD_UTIL = "systemd-util"
            SYSTEMD_SYSTEM_UNIT = "systemd-system-unit"
            SYSTEMD_SYSTEM_PRESET = "systemd-system-preset"
            SYSTEMD_SYSTEM_CONF = "systemd-system-conf"
            SYSTEMD_USER_UNIT = "systemd-user-unit"
            SYSTEMD_USER_PRESET = "systemd-user-preset"
            SYSTEMD_USER_CONF = "systemd-user-conf"
            SYSTEMD_SEARCH_SYSTEM_UNIT = "systemd-search-system-unit"
            SYSTEMD_SEARCH_USER_UNIT = "systemd-search-user-unit"
            SYSTEMD_SYSTEM_GENERATOR = "systemd-system-generator"
            SYSTEMD_USER_GENERATOR = "systemd-user-generator"
            SYSTEMD_SEARCH_SYSTEM_GENERATOR = "systemd-search-system-generator"
            SYSTEMD_SEARCH_USER_GENERATOR = "systemd-search-user-generator"
            SYSTEMD_SLEEP = "systemd-sleep"
            SYSTEMD_SHUTDOWN = "systemd-shutdown"
            TMPFILES = "tmpfiles"
            SYSUSERS = "sysusers"
            SYSCTL = "sysctl"
            BINFMT = "binfmt"
            MODULES_LOAD = "modules-load"
            CATALOG = "catalog"
            SYSTEMD_SEARCH_NETWORK = "systemd-search-network"
            SYSTEMD_SYSTEM_ENVIRONMENT_GENERATOR = \
                "systemd-system-environment-generator"
            SYSTEMD_USER_ENVIRONMENT_GENERATOR = \
                "systemd-user-environment-generator"
            SYSTEMD_SEARCH_SYSTEM_ENVIRONMENT_GENERATOR = \
                "systemd-search-system-environment-generator"
            SYSTEMD_SEARCH_USER_ENVIRONMENT_GENERATOR = \
                "systemd-search-user-environment-generator"

        def __init__(self):
            names = {i.value: i.name for i in self.table}
            self.__names: Dict[str, str] = names
            self.__items: Dict[str, Tuple[str, ...]] = dict()
            for line in os.popen(CMD_SD_PATH).readlines():
                name, path = line.split(":", 1)
                assert isinstance(name, str)
                assert isinstance(path, str)
                dirs: List[str] = path.strip().split(":")
                self.__items[name.strip()] = tuple([i.strip() for i in dirs])

        def __dir__(self):
            dirs: List[str] = dir(self.__class__)
            dirs.extend(self.__names.values())
            return dirs

        def __iter__(self):
            return iter(self.__items.keys())

        def __getitem__(self, name: str) -> Tuple[str, ...]:
            return self.__items[name]

        # def __getattr__(self, name: str) -> Tuple[str, ...]:
        #     if name not in self.__names.values():
        #         raise AttributeError(f"{CMD_SD_PATH} has no '{name}'")
        #     item: __sd_path.table = getattr(self.table, name)
        #     return self.__items[item.value]

        @property
        def systemd_system_conf_dir(self) -> str:
            return self[self.table.SYSTEMD_SYSTEM_CONF.value][0]

        @property
        def systemd_system_unit_dirs(self) -> Tuple[str, ...]:
            return self[self.table.SYSTEMD_SEARCH_SYSTEM_UNIT.value] \
                + self[self.table.SYSTEMD_SYSTEM_UNIT.value]

        @property
        def systemd_system_generator_dirs(self) -> Tuple[str, ...]:
            return self[self.table.SYSTEMD_SEARCH_SYSTEM_GENERATOR.value] \
                + self[self.table.SYSTEMD_SYSTEM_GENERATOR.value]

        @property
        def systemd_system_dirs(self) -> Tuple[str, ...]:
            return self[self.table.SYSTEMD_SYSTEM_CONF.value] \
                + self[self.table.SYSTEMD_SYSTEM_UNIT.value] \
                + self[self.table.SYSTEMD_SYSTEM_GENERATOR.value]

        @property
        def titles(self) -> List[str]:
            return list(self.__names.keys())

    sd_path = __sd_path()



# coding:utf-8

from configparser import ConfigParser
from configparser import SectionProxy
from datetime import datetime
from enum import Enum
import os
from typing import Optional
from typing import Set
from typing import Tuple

from xmanage.attribute import __project__
from xmanage.attribute import __version__
from xmanage.systemd.unit import sd_unit_file
from xmanage.systemd.unit import sd_unit_file_section

allowed_dirs: Optional[Tuple[str, ...]] = None
try:
    from xmanage.systemd.path import sd_path
    allowed_dirs = sd_path.systemd_system_dirs
except Exception:  # pragma: no cover
    pass  # pragma: no cover


class sd_uf_sec_service(sd_unit_file_section):
    """[Service] Section Options"""
    SECTION_NAME: str = "Service"

    # https://www.freedesktop.org/software/systemd/man/latest/systemd.service.html#Options
    class options(Enum):
        TYPE = "Type"
        EXITTYPE = "ExitType"
        REMAINAFTEREXIT = "RemainAfterExit"
        GUESSMAINPID = "GuessMainPID"
        PIDFILE = "PIDFile"
        BUSNAME = "BusName"
        EXECSTART = "ExecStart"
        EXECSTARTPRE = "ExecStartPre"
        EXECSTARTPOST = "ExecStartPost"
        EXECCONDITION = "ExecCondition"
        EXECRELOAD = "ExecReload"
        EXECSTOP = "ExecStop"
        EXECSTOPPOST = "ExecStopPost"
        RESTARTSEC = "RestartSec"
        RESTARTSTEPS = "RestartSteps"
        RESTARTMAXDELAYSEC = "RestartMaxDelaySec"
        TIMEOUTSTARTSEC = "TimeoutStartSec"
        TIMEOUTSTOPSEC = "TimeoutStopSec"
        TIMEOUTABORTSEC = "TimeoutAbortSec"
        TIMEOUTSEC = "TimeoutSec"
        TIMEOUTSTARTFAILUREMODE = "TimeoutStartFailureMode"
        TIMEOUTSTOPFAILUREMODE = "TimeoutStopFailureMode"
        RUNTIMEMAXSEC = "RuntimeMaxSec"
        RUNTIMERANDOMIZEDEXTRASEC = "RuntimeRandomizedExtraSec"
        WATCHDOGSEC = "WatchdogSec"
        RESTART = "Restart"
        RESTARTMODE = "RestartMode"
        SUCCESSEXITSTATUS = "SuccessExitStatus"
        RESTARTPREVENTEXITSTATUS = "RestartPreventExitStatus"
        RESTARTFORCEEXITSTATUS = "RestartForceExitStatus"
        ROOTDIRECTORYSTARTONLY = "RootDirectoryStartOnly"
        NONBLOCKING = "NonBlocking"
        NOTIFYACCESS = "NotifyAccess"
        SOCKETS = "Sockets"
        FILEDESCRIPTORSTOREMAX = "FileDescriptorStoreMax"
        FILEDESCRIPTORSTOREPRESERVE = "FileDescriptorStorePreserve"
        USBFUNCTIONDESCRIPTORS = "USBFunctionDescriptors"
        USBFUNCTIONSTRINGS = "USBFunctionStrings"
        OOMPOLICY = "OOMPolicy"
        OPENFILE = "OpenFile"
        RELOADSIGNAL = "ReloadSignal"
    OPTIONS: Set[str] = {i.value for i in options}

    def __init__(self, section: SectionProxy):
        super().__init__(self.SECTION_NAME, section)

    @property
    def Type(self) -> str:
        return self.get_option(self.options.TYPE.value)

    @Type.setter
    def Type(self, value: str):
        self.set_option(self.options.TYPE.value, value)

    @property
    def ExitType(self) -> str:
        return self.get_option(self.options.EXITTYPE.value)

    @ExitType.setter
    def ExitType(self, value: str):
        self.set_option(self.options.EXITTYPE.value, value)

    @property
    def RemainAfterExit(self) -> str:
        return self.get_option(self.options.REMAINAFTEREXIT.value)

    @RemainAfterExit.setter
    def RemainAfterExit(self, value: str):
        self.set_option(self.options.REMAINAFTEREXIT.value, value)

    @property
    def GuessMainPID(self) -> str:
        return self.get_option(self.options.GUESSMAINPID.value)

    @GuessMainPID.setter
    def GuessMainPID(self, value: str):
        self.set_option(self.options.GUESSMAINPID.value, value)

    @property
    def PIDFile(self) -> str:
        return self.get_option(self.options.PIDFILE.value)

    @PIDFile.setter
    def PIDFile(self, value: str):
        self.set_option(self.options.PIDFILE.value, value)

    @property
    def BusName(self) -> str:
        return self.get_option(self.options.BUSNAME.value)

    @BusName.setter
    def BusName(self, value: str):
        self.set_option(self.options.BUSNAME.value, value)

    @property
    def ExecStart(self) -> str:
        return self.get_option(self.options.EXECSTART.value)

    @ExecStart.setter
    def ExecStart(self, value: str):
        self.set_option(self.options.EXECSTART.value, value)

    @property
    def ExecStartPre(self) -> str:
        return self.get_option(self.options.EXECSTARTPRE.value)

    @ExecStartPre.setter
    def ExecStartPre(self, value: str):
        self.set_option(self.options.EXECSTARTPRE.value, value)

    @property
    def ExecStartPost(self) -> str:
        return self.get_option(self.options.EXECSTARTPOST.value)

    @ExecStartPost.setter
    def ExecStartPost(self, value: str):
        self.set_option(self.options.EXECSTARTPOST.value, value)

    @property
    def ExecCondition(self) -> str:
        return self.get_option(self.options.EXECCONDITION.value)

    @ExecCondition.setter
    def ExecCondition(self, value: str):
        self.set_option(self.options.EXECCONDITION.value, value)

    @property
    def ExecReload(self) -> str:
        return self.get_option(self.options.EXECRELOAD.value)

    @ExecReload.setter
    def ExecReload(self, value: str):
        self.set_option(self.options.EXECRELOAD.value, value)

    @property
    def ExecStop(self) -> str:
        return self.get_option(self.options.EXECSTOP.value)

    @ExecStop.setter
    def ExecStop(self, value: str):
        self.set_option(self.options.EXECSTOP.value, value)

    @property
    def ExecStopPost(self) -> str:
        return self.get_option(self.options.EXECSTOPPOST.value)

    @ExecStopPost.setter
    def ExecStopPost(self, value: str):
        self.set_option(self.options.EXECSTOPPOST.value, value)

    @property
    def RestartSec(self) -> str:
        return self.get_option(self.options.RESTARTSEC.value)

    @RestartSec.setter
    def RestartSec(self, value: str):
        self.set_option(self.options.RESTARTSEC.value, value)

    @property
    def RestartSteps(self) -> str:
        return self.get_option(self.options.RESTARTSTEPS.value)

    @RestartSteps.setter
    def RestartSteps(self, value: str):
        self.set_option(self.options.RESTARTSTEPS.value, value)

    @property
    def RestartMaxDelaySec(self) -> str:
        return self.get_option(self.options.RESTARTMAXDELAYSEC.value)

    @RestartMaxDelaySec.setter
    def RestartMaxDelaySec(self, value: str):
        self.set_option(self.options.RESTARTMAXDELAYSEC.value, value)

    @property
    def TimeoutStartSec(self) -> str:
        return self.get_option(self.options.TIMEOUTSTARTSEC.value)

    @TimeoutStartSec.setter
    def TimeoutStartSec(self, value: str):
        self.set_option(self.options.TIMEOUTSTARTSEC.value, value)

    @property
    def TimeoutStopSec(self) -> str:
        return self.get_option(self.options.TIMEOUTSTOPSEC.value)

    @TimeoutStopSec.setter
    def TimeoutStopSec(self, value: str):
        self.set_option(self.options.TIMEOUTSTOPSEC.value, value)

    @property
    def TimeoutAbortSec(self) -> str:
        return self.get_option(self.options.TIMEOUTABORTSEC.value)

    @TimeoutAbortSec.setter
    def TimeoutAbortSec(self, value: str):
        self.set_option(self.options.TIMEOUTABORTSEC.value, value)

    @property
    def TimeoutSec(self) -> str:
        return self.get_option(self.options.TIMEOUTSEC.value)

    @TimeoutSec.setter
    def TimeoutSec(self, value: str):
        self.set_option(self.options.TIMEOUTSEC.value, value)

    @property
    def TimeoutStartFailureMode(self) -> str:
        return self.get_option(self.options.TIMEOUTSTARTFAILUREMODE.value)

    @TimeoutStartFailureMode.setter
    def TimeoutStartFailureMode(self, value: str):
        self.set_option(self.options.TIMEOUTSTARTFAILUREMODE.value, value)

    @property
    def TimeoutStopFailureMode(self) -> str:
        return self.get_option(self.options.TIMEOUTSTOPFAILUREMODE.value)

    @TimeoutStopFailureMode.setter
    def TimeoutStopFailureMode(self, value: str):
        self.set_option(self.options.TIMEOUTSTOPFAILUREMODE.value, value)

    @property
    def RuntimeMaxSec(self) -> str:
        return self.get_option(self.options.RUNTIMEMAXSEC.value)

    @RuntimeMaxSec.setter
    def RuntimeMaxSec(self, value: str):
        self.set_option(self.options.RUNTIMEMAXSEC.value, value)

    @property
    def RuntimeRandomizedExtraSec(self) -> str:
        return self.get_option(self.options.RUNTIMERANDOMIZEDEXTRASEC.value)

    @RuntimeRandomizedExtraSec.setter
    def RuntimeRandomizedExtraSec(self, value: str):
        self.set_option(self.options.RUNTIMERANDOMIZEDEXTRASEC.value, value)

    @property
    def WatchdogSec(self) -> str:
        return self.get_option(self.options.WATCHDOGSEC.value)

    @WatchdogSec.setter
    def WatchdogSec(self, value: str):
        self.set_option(self.options.WATCHDOGSEC.value, value)

    @property
    def Restart(self) -> str:
        return self.get_option(self.options.RESTART.value)

    @Restart.setter
    def Restart(self, value: str):
        self.set_option(self.options.RESTART.value, value)

    @property
    def RestartMode(self) -> str:
        return self.get_option(self.options.RESTARTMODE.value)

    @RestartMode.setter
    def RestartMode(self, value: str):
        self.set_option(self.options.RESTARTMODE.value, value)

    @property
    def SuccessExitStatus(self) -> str:
        return self.get_option(self.options.SUCCESSEXITSTATUS.value)

    @SuccessExitStatus.setter
    def SuccessExitStatus(self, value: str):
        self.set_option(self.options.SUCCESSEXITSTATUS.value, value)

    @property
    def RestartPreventExitStatus(self) -> str:
        return self.get_option(self.options.RESTARTPREVENTEXITSTATUS.value)

    @RestartPreventExitStatus.setter
    def RestartPreventExitStatus(self, value: str):
        self.set_option(self.options.RESTARTPREVENTEXITSTATUS.value, value)

    @property
    def RestartForceExitStatus(self) -> str:
        return self.get_option(self.options.RESTARTFORCEEXITSTATUS.value)

    @RestartForceExitStatus.setter
    def RestartForceExitStatus(self, value: str):
        self.set_option(self.options.RESTARTFORCEEXITSTATUS.value, value)

    @property
    def RootDirectoryStartOnly(self) -> str:
        return self.get_option(self.options.ROOTDIRECTORYSTARTONLY.value)

    @RootDirectoryStartOnly.setter
    def RootDirectoryStartOnly(self, value: str):
        self.set_option(self.options.ROOTDIRECTORYSTARTONLY.value, value)

    @property
    def NonBlocking(self) -> str:
        return self.get_option(self.options.NONBLOCKING.value)

    @NonBlocking.setter
    def NonBlocking(self, value: str):
        self.set_option(self.options.NONBLOCKING.value, value)

    @property
    def NotifyAccess(self) -> str:
        return self.get_option(self.options.NOTIFYACCESS.value)

    @NotifyAccess.setter
    def NotifyAccess(self, value: str):
        self.set_option(self.options.NOTIFYACCESS.value, value)

    @property
    def Sockets(self) -> str:
        return self.get_option(self.options.SOCKETS.value)

    @Sockets.setter
    def Sockets(self, value: str):
        self.set_option(self.options.SOCKETS.value, value)

    @property
    def FileDescriptorStoreMax(self) -> str:
        return self.get_option(self.options.FILEDESCRIPTORSTOREMAX.value)

    @FileDescriptorStoreMax.setter
    def FileDescriptorStoreMax(self, value: str):
        self.set_option(self.options.FILEDESCRIPTORSTOREMAX.value, value)

    @property
    def FileDescriptorStorePreserve(self) -> str:
        return self.get_option(self.options.FILEDESCRIPTORSTOREPRESERVE.value)

    @FileDescriptorStorePreserve.setter
    def FileDescriptorStorePreserve(self, value: str):
        self.set_option(self.options.FILEDESCRIPTORSTOREPRESERVE.value, value)

    @property
    def USBFunctionDescriptors(self) -> str:
        return self.get_option(self.options.USBFUNCTIONDESCRIPTORS.value)

    @USBFunctionDescriptors.setter
    def USBFunctionDescriptors(self, value: str):
        self.set_option(self.options.USBFUNCTIONDESCRIPTORS.value, value)

    @property
    def USBFunctionStrings(self) -> str:
        return self.get_option(self.options.USBFUNCTIONSTRINGS.value)

    @USBFunctionStrings.setter
    def USBFunctionStrings(self, value: str):
        self.set_option(self.options.USBFUNCTIONSTRINGS.value, value)

    @property
    def OOMPolicy(self) -> str:
        return self.get_option(self.options.OOMPOLICY.value)

    @OOMPolicy.setter
    def OOMPolicy(self, value: str):
        self.set_option(self.options.OOMPOLICY.value, value)

    @property
    def OpenFile(self) -> str:
        return self.get_option(self.options.OPENFILE.value)

    @OpenFile.setter
    def OpenFile(self, value: str):
        self.set_option(self.options.OPENFILE.value, value)

    @property
    def ReloadSignal(self) -> str:
        return self.get_option(self.options.RELOADSIGNAL.value)

    @ReloadSignal.setter
    def ReloadSignal(self, value: str):
        self.set_option(self.options.RELOADSIGNAL.value, value)


class sd_service(sd_unit_file):

    def __init__(self, config: ConfigParser):
        super().__init__(config)
        _svc: SectionProxy = self.get_section(sd_uf_sec_service.SECTION_NAME)
        self.__section_service: sd_uf_sec_service = sd_uf_sec_service(_svc)

    @property
    def service_section(self) -> sd_uf_sec_service:
        return self.__section_service

    @classmethod
    def from_string(cls, value: str) -> "sd_service":
        assert isinstance(value, str), f"unexpected type: {type(value)}"
        return sd_service(cls.read_string(value))

    @classmethod
    def from_file(cls, file: str) -> "sd_service":
        assert os.path.exists(file), f"'{file}' not found."
        assert os.path.isfile(file), f"'{file}' is not a regular file."
        return sd_service.from_string(open(file).read())

    @classmethod
    def format_service_unit(cls, unit: str) -> str:
        return unit if unit.endswith(".service") else ".".join([unit, "service"])  # noqa:E501

    @classmethod
    def service_unit_path(cls, unit: str, folder: Optional[str] = None) -> str:
        if folder is None:
            folder = sd_path.systemd_system_conf_dir

        assert isinstance(unit, str), f"unexpected type: {type(unit)}"
        assert isinstance(folder, str), f"unexpected type: {type(folder)}"

        if allowed_dirs is not None:
            assert folder in allowed_dirs, f"'{folder}' is not in {allowed_dirs}"  # noqa:E501

        assert os.path.isdir(folder), f"'{folder}' not exists or is file"
        return os.path.join(folder, cls.format_service_unit(unit))

    def create_unit(self, unit: str, folder: Optional[str] = None,
                    allow_update: bool = False) -> str:
        unitname: str = self.format_service_unit(unit)
        filepath: str = self.service_unit_path(unitname, folder)
        if os.path.exists(filepath) and not allow_update:
            raise FileExistsError(f"'{filepath}' already exists.")

        with open(filepath, "w") as hdl:
            now: datetime = datetime.now()
            hdl.write(f"# {unitname} created by {__project__} {__version__}\n")
            hdl.write(f"# {now.strftime('%a %b %d %H:%M:%S CST %Y')}\n\n\n")
            self.parser.write(hdl)

        os.system("systemctl daemon-reload")
        return filepath

    @classmethod
    def delete_unit(cls, unit: str, folder: Optional[str] = None) -> bool:
        unitname: str = cls.format_service_unit(unit)
        filepath: str = cls.service_unit_path(unitname, folder)

        if os.path.isfile(filepath):
            os.remove(filepath)

        os.system("systemctl daemon-reload")
        return not os.path.exists(filepath)



# coding:utf-8

from typing import List
from typing import Optional
from typing import Sequence

from xkits_command import ArgParser
from xkits_command import Command
from xkits_command import CommandArgument
from xkits_command import CommandExecutor

from xmanage.attribute import __description__
from xmanage.attribute import __project__
from xmanage.attribute import __urlhome__
from xmanage.attribute import __version__

subs: List[CommandArgument] = list()

try:
    from xmanage.xsystem.path import add_cmd_path
    subs.append(add_cmd_path)
except Exception:  # pragma: no cover
    pass  # pragma: no cover


@CommandArgument(__project__)
def add_cmd(_arg: ArgParser):
    pass


@CommandExecutor(add_cmd, *subs)
def run_cmd(cmds: Command) -> int:
    return 0


def main(argv: Optional[Sequence[str]] = None) -> int:
    cmds = Command()
    cmds.version = __version__
    return cmds.run(
        root=add_cmd,
        argv=argv,
        description=__description__,
        epilog=f"For more, please visit {__urlhome__}.")



# coding:utf-8

from xmanage.systemd import systemd_service  # noqa:F401

try:
    from xmanage.systemd import systemd_path  # noqa:F401
except Exception:  # pragma: no cover pylint: disable=broad-exception-caught
    pass  # pragma: no cover



# coding=utf-8

from urllib.parse import urljoin

from setuptools import find_packages
from setuptools import setup

from xmanage.attribute import __author__
from xmanage.attribute import __author_email__
from xmanage.attribute import __description__
from xmanage.attribute import __project__
from xmanage.attribute import __urlhome__
from xmanage.attribute import __version__

__urlcode__ = __urlhome__
__urldocs__ = __urlhome__
__urlbugs__ = urljoin(__urlhome__, "issues")


def all_requirements():
    def read_requirements(path: str):
        with open(path, "r", encoding="utf-8") as rhdl:
            return rhdl.read().splitlines()

    requirements = read_requirements("requirements.txt")
    return requirements


setup(
    name=__project__,
    version=__version__,
    description=__description__,
    url=__urlhome__,
    author=__author__,
    author_email=__author_email__,
    project_urls={"Source Code": __urlcode__,
                  "Bug Tracker": __urlbugs__,
                  "Documentation": __urldocs__},
    packages=find_packages(include=["xmanage*"], exclude=["xmanage.unittest"]),
    install_requires=all_requirements())



# coding:utf-8

from configparser import ConfigParser
from configparser import SectionProxy
from enum import Enum
from typing import Any
from typing import ItemsView
from typing import KeysView
from typing import List
from typing import Optional
from typing import Set
from typing import Union
from typing import ValuesView


class sd_unit_file_section:
    """unit file section"""

    def __init__(self, name: str, section: SectionProxy):
        assert isinstance(name, str), f"unexpected type: {type(name)}"
        assert isinstance(section, SectionProxy), \
            f"unexpected type: {type(section)}"
        self.__section_name: str = name
        self.__section: SectionProxy = section

    def __iter__(self):
        return iter(self.items)

    # def __getattr__(self, attr: str) -> Any:
    #     try:
    #         return object.__getattribute__(self, attr)
    #     except AttributeError as e:
    #         if attr in object.__getattribute__(self, "OPTIONS"):
    #             return self.get_option(attr)
    #         raise e

    # def __setattr__(self, attr: str, value: str) -> None:
    #     try:
    #         if attr in self.__getattribute__("OPTIONS"):
    #             return self.set_option(attr, value)
    #     except AttributeError:
    #         pass
    #     return super().__setattr__(attr, value)

    @property
    def name(self) -> str:
        return self.__section_name

    @property
    def items(self) -> ItemsView[str, str]:
        return self.__section.items()

    @property
    def keys(self) -> KeysView[str]:
        return self.__section.keys()

    @property
    def values(self) -> ValuesView[str]:
        return self.__section.values()

    def get_option(self, option: str, fallback: Optional[str] = "") -> Union[str, Any]:  # noqa:E501
        """Get an option value."""
        return self.__section.get(option, fallback)

    def set_option(self, option: str, value: str):
        """Set value for an option."""
        self.__section[option] = value

    def setdefault(self, option: str, default: str) -> str:
        """Set default value for an option."""
        return self.__section.setdefault(option, default)


class sd_uf_sec_unit(sd_unit_file_section):
    """[Unit] Section Options"""
    SECTION_NAME: str = "Unit"

    # https://www.freedesktop.org/software/systemd/man/latest/systemd.unit.html#%5BUnit%5D%20Section%20Options
    class options(Enum):
        DESCRIPTION = "Description"
        DOCUMENTATION = "Documentation"
        WANTS = "Wants"
        REQUIRES = "Requires"
        REQUISITE = "Requisite"
        BINDSTO = "BindsTo"
        PARTOF = "PartOf"
        UPHOLDS = "Upholds"
        CONFLICTS = "Conflicts"
        BEFORE = "Before"
        AFTER = "After"
        ONFAILURE = "OnFailure"
        ONSUCCESS = "OnSuccess"
        PROPAGATESRELOADTO = "PropagatesReloadTo"
        RELOADPROPAGATEDFROM = "ReloadPropagatedFrom"
        PROPAGATESSTOPTO = "PropagatesStopTo"
        STOPPROPAGATEDFROM = "StopPropagatedFrom"
        JOINSNAMESPACEOF = "JoinsNamespaceOf"
        REQUIRESMOUNTSFOR = "RequiresMountsFor"
        ONSUCCESSJOBMODE = "OnSuccessJobMode"
        ONFAILUREJOBMODE = "OnFailureJobMode"
        IGNOREONISOLATE = "IgnoreOnIsolate"
        STOPWHENUNNEEDED = "StopWhenUnneeded"
        REFUSEMANUALSTART = "RefuseManualStart"
        REFUSEMANUALSTOP = "RefuseManualStop"
        ALLOWISOLATE = "AllowIsolate"
        DEFAULTDEPENDENCIES = "DefaultDependencies"
        SURVIVEFINALKILLSIGNAL = "SurviveFinalKillSignal"
        COLLECTMODE = "CollectMode"
        FAILUREACTION = "FailureAction"
        SUCCESSACTION = "SuccessAction"
        FAILUREACTIONEXITSTATUS = "FailureActionExitStatus"
        SUCCESSACTIONEXITSTATUS = "SuccessActionExitStatus"
        JOBTIMEOUTSEC = "JobTimeoutSec"
        JOBRUNNINGTIMEOUTSEC = "JobRunningTimeoutSec"
        JOBTIMEOUTACTION = "JobTimeoutAction"
        JOBTIMEOUTREBOOTARGUMENT = "JobTimeoutRebootArgument"
        STARTLIMITINTERVALSEC = "StartLimitIntervalSec"
        STARTLIMITBURST = "StartLimitBurst"
        STARTLIMITACTION = "StartLimitAction"
        REBOOTARGUMENT = "RebootArgument"
        SOURCEPATH = "SourcePath"
    OPTIONS: Set[str] = {i.value for i in options}

    def __init__(self, section: SectionProxy):
        super().__init__(self.SECTION_NAME, section)

    @property
    def Description(self) -> str:
        return self.get_option(self.options.DESCRIPTION.value)

    @Description.setter
    def Description(self, value: str):
        self.set_option(self.options.DESCRIPTION.value, value)

    @property
    def Documentation(self) -> str:
        return self.get_option(self.options.DOCUMENTATION.value)

    @Documentation.setter
    def Documentation(self, value: str):
        self.set_option(self.options.DOCUMENTATION.value, value)

    @property
    def Wants(self) -> str:
        return self.get_option(self.options.WANTS.value)

    @Wants.setter
    def Wants(self, value: str):
        self.set_option(self.options.WANTS.value, value)

    @property
    def Requires(self) -> str:
        return self.get_option(self.options.REQUIRES.value)

    @Requires.setter
    def Requires(self, value: str):
        self.set_option(self.options.REQUIRES.value, value)

    @property
    def Requisite(self) -> str:
        return self.get_option(self.options.REQUISITE.value)

    @Requisite.setter
    def Requisite(self, value: str):
        self.set_option(self.options.REQUISITE.value, value)

    @property
    def BindsTo(self) -> str:
        return self.get_option(self.options.BINDSTO.value)

    @BindsTo.setter
    def BindsTo(self, value: str):
        self.set_option(self.options.BINDSTO.value, value)

    @property
    def PartOf(self) -> str:
        return self.get_option(self.options.PARTOF.value)

    @PartOf.setter
    def PartOf(self, value: str):
        self.set_option(self.options.PARTOF.value, value)

    @property
    def Upholds(self) -> str:
        return self.get_option(self.options.UPHOLDS.value)

    @Upholds.setter
    def Upholds(self, value: str):
        self.set_option(self.options.UPHOLDS.value, value)

    @property
    def Conflicts(self) -> str:
        return self.get_option(self.options.CONFLICTS.value)

    @Conflicts.setter
    def Conflicts(self, value: str):
        self.set_option(self.options.CONFLICTS.value, value)

    @property
    def Before(self) -> str:
        return self.get_option(self.options.BEFORE.value)

    @Before.setter
    def Before(self, value: str):
        self.set_option(self.options.BEFORE.value, value)

    @property
    def After(self) -> str:
        return self.get_option(self.options.AFTER.value)

    @After.setter
    def After(self, value: str):
        self.set_option(self.options.AFTER.value, value)

    @property
    def OnFailure(self) -> str:
        return self.get_option(self.options.ONFAILURE.value)

    @OnFailure.setter
    def OnFailure(self, value: str):
        self.set_option(self.options.ONFAILURE.value, value)

    @property
    def OnSuccess(self) -> str:
        return self.get_option(self.options.ONSUCCESS.value)

    @OnSuccess.setter
    def OnSuccess(self, value: str):
        self.set_option(self.options.ONSUCCESS.value, value)

    @property
    def PropagatesReloadTo(self) -> str:
        return self.get_option(self.options.PROPAGATESRELOADTO.value)

    @PropagatesReloadTo.setter
    def PropagatesReloadTo(self, value: str):
        self.set_option(self.options.PROPAGATESRELOADTO.value, value)

    @property
    def ReloadPropagatedFrom(self) -> str:
        return self.get_option(self.options.RELOADPROPAGATEDFROM.value)

    @ReloadPropagatedFrom.setter
    def ReloadPropagatedFrom(self, value: str):
        self.set_option(self.options.RELOADPROPAGATEDFROM.value, value)

    @property
    def PropagatesStopTo(self) -> str:
        return self.get_option(self.options.PROPAGATESSTOPTO.value)

    @PropagatesStopTo.setter
    def PropagatesStopTo(self, value: str):
        self.set_option(self.options.PROPAGATESSTOPTO.value, value)

    @property
    def StopPropagatedFrom(self) -> str:
        return self.get_option(self.options.STOPPROPAGATEDFROM.value)

    @StopPropagatedFrom.setter
    def StopPropagatedFrom(self, value: str):
        self.set_option(self.options.STOPPROPAGATEDFROM.value, value)

    @property
    def JoinsNamespaceOf(self) -> str:
        return self.get_option(self.options.JOINSNAMESPACEOF.value)

    @JoinsNamespaceOf.setter
    def JoinsNamespaceOf(self, value: str):
        self.set_option(self.options.JOINSNAMESPACEOF.value, value)

    @property
    def RequiresMountsFor(self) -> str:
        return self.get_option(self.options.REQUIRESMOUNTSFOR.value)

    @RequiresMountsFor.setter
    def RequiresMountsFor(self, value: str):
        self.set_option(self.options.REQUIRESMOUNTSFOR.value, value)

    @property
    def OnSuccessJobMode(self) -> str:
        return self.get_option(self.options.ONSUCCESSJOBMODE.value)

    @OnSuccessJobMode.setter
    def OnSuccessJobMode(self, value: str):
        self.set_option(self.options.ONSUCCESSJOBMODE.value, value)

    @property
    def OnFailureJobMode(self) -> str:
        return self.get_option(self.options.ONFAILUREJOBMODE.value)

    @OnFailureJobMode.setter
    def OnFailureJobMode(self, value: str):
        self.set_option(self.options.ONFAILUREJOBMODE.value, value)

    @property
    def IgnoreOnIsolate(self) -> str:
        return self.get_option(self.options.IGNOREONISOLATE.value)

    @IgnoreOnIsolate.setter
    def IgnoreOnIsolate(self, value: str):
        self.set_option(self.options.IGNOREONISOLATE.value, value)

    @property
    def StopWhenUnneeded(self) -> str:
        return self.get_option(self.options.STOPWHENUNNEEDED.value)

    @StopWhenUnneeded.setter
    def StopWhenUnneeded(self, value: str):
        self.set_option(self.options.STOPWHENUNNEEDED.value, value)

    @property
    def RefuseManualStart(self) -> str:
        return self.get_option(self.options.REFUSEMANUALSTART.value)

    @RefuseManualStart.setter
    def RefuseManualStart(self, value: str):
        self.set_option(self.options.REFUSEMANUALSTART.value, value)

    @property
    def RefuseManualStop(self) -> str:
        return self.get_option(self.options.REFUSEMANUALSTOP.value)

    @RefuseManualStop.setter
    def RefuseManualStop(self, value: str):
        self.set_option(self.options.REFUSEMANUALSTOP.value, value)

    @property
    def AllowIsolate(self) -> str:
        return self.get_option(self.options.ALLOWISOLATE.value)

    @AllowIsolate.setter
    def AllowIsolate(self, value: str):
        self.set_option(self.options.ALLOWISOLATE.value, value)

    @property
    def DefaultDependencies(self) -> str:
        return self.get_option(self.options.DEFAULTDEPENDENCIES.value)

    @DefaultDependencies.setter
    def DefaultDependencies(self, value: str):
        self.set_option(self.options.DEFAULTDEPENDENCIES.value, value)

    @property
    def SurviveFinalKillSignal(self) -> str:
        return self.get_option(self.options.SURVIVEFINALKILLSIGNAL.value)

    @SurviveFinalKillSignal.setter
    def SurviveFinalKillSignal(self, value: str):
        self.set_option(self.options.SURVIVEFINALKILLSIGNAL.value, value)

    @property
    def CollectMode(self) -> str:
        return self.get_option(self.options.COLLECTMODE.value)

    @CollectMode.setter
    def CollectMode(self, value: str):
        self.set_option(self.options.COLLECTMODE.value, value)

    @property
    def FailureAction(self) -> str:
        return self.get_option(self.options.FAILUREACTION.value)

    @FailureAction.setter
    def FailureAction(self, value: str):
        self.set_option(self.options.FAILUREACTION.value, value)

    @property
    def SuccessAction(self) -> str:
        return self.get_option(self.options.SUCCESSACTION.value)

    @SuccessAction.setter
    def SuccessAction(self, value: str):
        self.set_option(self.options.SUCCESSACTION.value, value)

    @property
    def FailureActionExitStatus(self) -> str:
        return self.get_option(self.options.FAILUREACTIONEXITSTATUS.value)

    @FailureActionExitStatus.setter
    def FailureActionExitStatus(self, value: str):
        self.set_option(self.options.FAILUREACTIONEXITSTATUS.value, value)

    @property
    def SuccessActionExitStatus(self) -> str:
        return self.get_option(self.options.SUCCESSACTIONEXITSTATUS.value)

    @SuccessActionExitStatus.setter
    def SuccessActionExitStatus(self, value: str):
        self.set_option(self.options.SUCCESSACTIONEXITSTATUS.value, value)

    @property
    def JobTimeoutSec(self) -> str:
        return self.get_option(self.options.JOBTIMEOUTSEC.value)

    @JobTimeoutSec.setter
    def JobTimeoutSec(self, value: str):
        self.set_option(self.options.JOBTIMEOUTSEC.value, value)

    @property
    def JobRunningTimeoutSec(self) -> str:
        return self.get_option(self.options.JOBRUNNINGTIMEOUTSEC.value)

    @JobRunningTimeoutSec.setter
    def JobRunningTimeoutSec(self, value: str):
        self.set_option(self.options.JOBRUNNINGTIMEOUTSEC.value, value)

    @property
    def JobTimeoutAction(self) -> str:
        return self.get_option(self.options.JOBTIMEOUTACTION.value)

    @JobTimeoutAction.setter
    def JobTimeoutAction(self, value: str):
        self.set_option(self.options.JOBTIMEOUTACTION.value, value)

    @property
    def JobTimeoutRebootArgument(self) -> str:
        return self.get_option(self.options.JOBTIMEOUTREBOOTARGUMENT.value)

    @JobTimeoutRebootArgument.setter
    def JobTimeoutRebootArgument(self, value: str):
        self.set_option(self.options.JOBTIMEOUTREBOOTARGUMENT.value, value)

    @property
    def StartLimitIntervalSec(self) -> str:
        return self.get_option(self.options.STARTLIMITINTERVALSEC.value)

    @StartLimitIntervalSec.setter
    def StartLimitIntervalSec(self, value: str):
        self.set_option(self.options.STARTLIMITINTERVALSEC.value, value)

    @property
    def StartLimitBurst(self) -> str:
        return self.get_option(self.options.STARTLIMITBURST.value)

    @StartLimitBurst.setter
    def StartLimitBurst(self, value: str):
        self.set_option(self.options.STARTLIMITBURST.value, value)

    @property
    def StartLimitAction(self) -> str:
        return self.get_option(self.options.STARTLIMITACTION.value)

    @StartLimitAction.setter
    def StartLimitAction(self, value: str):
        self.set_option(self.options.STARTLIMITACTION.value, value)

    @property
    def RebootArgument(self) -> str:
        return self.get_option(self.options.REBOOTARGUMENT.value)

    @RebootArgument.setter
    def RebootArgument(self, value: str):
        self.set_option(self.options.REBOOTARGUMENT.value, value)

    @property
    def SourcePath(self) -> str:
        return self.get_option(self.options.SOURCEPATH.value)

    @SourcePath.setter
    def SourcePath(self, value: str):
        self.set_option(self.options.SOURCEPATH.value, value)


class sd_uf_sec_install(sd_unit_file_section):
    """[Install] Section Options"""
    SECTION_NAME: str = "Install"

    # https://www.freedesktop.org/software/systemd/man/latest/systemd.unit.html#%5BInstall%5D%20Section%20Options
    class options(Enum):
        ALIAS = "Alias"
        WANTEDBY = "WantedBy"
        REQUIREDBY = "RequiredBy"
        UPHELDBY = "UpheldBy"
        ALSO = "Also"
        DEFAULTINSTANCE = "DefaultInstance"
    OPTIONS: Set[str] = {i.value for i in options}

    def __init__(self, section: SectionProxy):
        super().__init__(self.SECTION_NAME, section)

    @property
    def Alias(self) -> str:
        return self.get_option(self.options.ALIAS.value)

    @Alias.setter
    def Alias(self, value: str):
        self.set_option(self.options.ALIAS.value, value)

    @property
    def WantedBy(self) -> str:
        return self.get_option(self.options.WANTEDBY.value)

    @WantedBy.setter
    def WantedBy(self, value: str):
        self.set_option(self.options.WANTEDBY.value, value)

    @property
    def RequiredBy(self) -> str:
        return self.get_option(self.options.REQUIREDBY.value)

    @RequiredBy.setter
    def RequiredBy(self, value: str):
        self.set_option(self.options.REQUIREDBY.value, value)

    @property
    def UpheldBy(self) -> str:
        return self.get_option(self.options.UPHELDBY.value)

    @UpheldBy.setter
    def UpheldBy(self, value: str):
        self.set_option(self.options.UPHELDBY.value, value)

    @property
    def Also(self) -> str:
        return self.get_option(self.options.ALSO.value)

    @Also.setter
    def Also(self, value: str):
        self.set_option(self.options.ALSO.value, value)

    @property
    def DefaultInstance(self) -> str:
        return self.get_option(self.options.DEFAULTINSTANCE.value)

    @DefaultInstance.setter
    def DefaultInstance(self, value: str):
        self.set_option(self.options.DEFAULTINSTANCE.value, value)


class sd_unit_file:
    """A plain text ini-style unit file."""

    def __init__(self, parser: ConfigParser):
        assert isinstance(parser, ConfigParser), \
            f"unexpected type: {type(parser)}"
        self.__parser: ConfigParser = parser
        _uni: SectionProxy = self.get_section(sd_uf_sec_unit.SECTION_NAME)
        _ins: SectionProxy = self.get_section(sd_uf_sec_install.SECTION_NAME)
        self.__section_unit: sd_uf_sec_unit = sd_uf_sec_unit(_uni)
        self.__section_install: sd_uf_sec_install = sd_uf_sec_install(_ins)

    @property
    def parser(self) -> ConfigParser:
        return self.__parser

    @property
    def sections(self) -> List[str]:
        return self.__parser.sections()

    @property
    def unit_section(self) -> sd_uf_sec_unit:
        return self.__section_unit

    @property
    def install_section(self) -> sd_uf_sec_install:
        return self.__section_install

    @classmethod
    def read_string(cls, value: str) -> ConfigParser:
        config: ConfigParser = ConfigParser()
        config.optionxform = lambda option: option  # type: ignore
        config.read_string(value)
        return config

    def get_section(self, section: str) -> SectionProxy:
        """Get a named section.

        Create a new section if it not exist.
        """
        if not self.__parser.has_section(section):
            self.__parser.add_section(section)
        return self.__parser[section]
